@@ -1,8 +1,9 @@
 ## Function Description:
 ##     This function removes all variables except those which are specified in
-##     the given character vector or regular expression.
+##     the given character vector, regular expression or both.
 
-rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limit=100, regex="auto"){
+rm.all.but <- function(keep = NULL, envir = .GlobalEnv, keep_functions = TRUE,
+                       gc_limit = 100, regex = "auto"){
     #----[ checking the input ]----#
     {
         ## Check the envir attribute
@@ -10,34 +11,40 @@ rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limi
             stop("You should specify an existing environment")
         }
 
+
         ## ckeck if keep is defined
         if (is.null(keep)) {
             stop("The parameter `keep` is not defined. It should be a chacter vector with length 1 or more.")
-        # if the provided object for keep is not a character vector
+            # if the provided object for keep is not a character vector
         } else if(class(keep) != "character") {
             stop("The parameter `keep` should be a chacter vector with length 1 or more.")
         }
 
+
         ## Check if the keep is a character vector
         if (class(keep) != "character" | typeof(keep) != "character" ) {
             stop("The value of `keep` parameter should be a chacter vector with length 1 or more.")
         }
 
+
         ## check if the length of keep is more than or equal to 1
         if (!length(keep)) {
             stop("The value of `keep` parameter should be a chacter vector with length 1 or more.")
         }
 
+
         ## if the keepFunctions is not a logical vector
         if (!is.logical(keep_functions) | length(keep_functions) != 1) {
             stop("The value of the `keepFunctions` should ve either TRUE or FLASE.")
         }
 
+
         ## check if the gc_limit has a valid value
         if (!is.numeric(gc_limit) | length(gc_limit) != 1) {
             stop("The value of `gc_limit` parameter should be numeric with length 1. It's unit is MB.")
         }
 
+
         ## check if the regex has a valid value
         if (!is.element(regex, c(TRUE, FALSE, "auto", 0, 1))) {
             stop("The value of `regex` should be either TRUE, FALSE or \"auto\".")
@@ -58,6 +65,8 @@ rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limi
                 ## remove regular expression pettern(s) from keep and put them in separate variable
                 regex_patterns <- keep[regex_index]
                 keep <- keep[-regex_index]
+            }else{
+                regex <- FALSE
             }
 
         # if user manually force the code not to concider regulat expression
@@ -93,6 +102,7 @@ rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limi
                 }
             }
 
+
             #----[ regex patterns ]----#
             {
                 ## check if there is any regex_patterns that does not match to anything
@@ -109,37 +119,44 @@ rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limi
                 }
             }
 
+
             # if there is any bad input
-            if (length(bad_input)>0) {
+            if (length(bad_input) > 0) {
                 # complain to user
-                stop(paste("All the items in the keep should be a real existing variable.\nThe following is/are not among variables of selected environment or patterns that match anything!\n", bad_input, sep = " "))
+                stop(paste("All the items in the keep should be a real existing variable or valid regular expressions.\nThe following is/are not among variables of selected environment or patterns that match anything!\n", bad_input, sep = " "))
             }
         }
 
         # initialise a variable that contains all the possible variables
         removables <- var_list
-
-
+        
+        
+        # if user have used a regular expression
+        if (regex) {
+            ## apply the regex in the following lines
+            # iterate through the regex patterns
+            for (i in regex_patterns) {
+                # get indices of items in removables that match the ith pattern
+                tmp_pattern_remove_list <- grep(pattern = i, x = removables)
+                # if there was any matches based on the pattern
+                if (length(tmp_pattern_remove_list)) {
+                    # remove the variables from the removables vector
+                    removables <- removables[-tmp_pattern_remove_list]
+                }
+            }
+        }
+        
+        
         # if there is anything left in keep variable
         if (length(keep)) {
             # list the name of variables that should be removed
             removables <- removables[!(removables %in% keep)]
+            
+            # avoid any duplicates which might appear by having regexp and having normal variable names
+            removables <- unique(removables)
         }
-
-
-        ## apply the regex in the following lines
-        # iterate through the regex patterns
-        for (i in regex_patterns) {
-            # get indices of items in removables that match the ith pattern
-            tmp_pattern_remove_list <- grep(pattern = i, x = removables)
-            # if there was any matches based on the pattern
-            if (length(tmp_pattern_remove_list)) {
-                # remove the variables from the removables vector
-                removables <- removables[-tmp_pattern_remove_list]
-            }
-        }
-
-
+        
+        
         # if anything has left to be removed
         if (length(removables)) {
             # get to total sum of the variables that are going to be removed in bytes
@@ -156,6 +173,8 @@ rm.all.but <- function(keep=NULL, envir=.GlobalEnv, keep_functions=TRUE, gc_limi
                 # call garbage collection
                 gc()
             }
+        }else{
+            warning("Nothing is left to be removed!")
         }
     }
 }
